#define __OCICPP_INTERNAL_USE_
#include "Cursor.h"


/* Construction/Destruction related methods */

Cursor::Cursor(OCIEnv *env,OCISvcCtx *svc,OCIError *err,int prefetch,int Type=DEFAULT) {
	init(env,svc,err,prefetch,Type);
}

Cursor::Cursor() {
	envhp=0; svchp=0; stmthp=0; paramd=0; errhp=0; nCols=0;
	nulltext=default_null_text; row=0; cursorType=DEFAULT;
	curRow=0; prefetchRows=0; fetched=0; fetched_all=0;
}

void Cursor::init(OCIEnv *env,OCISvcCtx *svc,OCIError *err,int prefetch,int Type=DEFAULT) {
	envhp=env; svchp=svc; stmthp=0; paramd=0; errhp=err;curRow=0; fetched=0;
	prefetchRows=prefetch; nulltext=default_null_text; row=0; nCols=0; cursorType=Type;
	fetched_all=0;
	//Alloc statement handle
	if(cursorType!=NTABLE) {
		status=OCIHandleAlloc( (dvoid *) envhp, (dvoid **) &stmthp, OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0);
		if(status!=OCI_SUCCESS) throw OraError("Handle Alloc Failed File: "__FILE__,status);
	}
	Initialized=1;
}

Cursor::~Cursor() {
	drop();	
}

void Cursor::drop() {
	if(!Initialized) return;
	map<string,OCIBind *>::const_iterator ci=binds.begin();
	while(ci!=binds.end()) {
		OCIHandleFree(ci->second,OCI_HTYPE_BIND);
		*ci++;
		DEBUG(DLEV_DEBUG,"Bind nandle freed\n");
	} 
	for(int i=0;i<nCols;i++) {
		delete row[i];
	}
	delete row;
	/* We have no need to free cause it will be done in OraRefCur */
	if(stmthp && cursorType!=NTABLE) OCIHandleFree(stmthp,OCI_HTYPE_STMT);
	Initialized=0;
}

/* --------------------------------------------------- */

void Cursor::execute(int prefetch=1) {
	prefetchRows=prefetch;
	if(cursorType==DEFAULT) {
		exec();
	} else {
		CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &stmtType,(ub4 *) 0,OCI_ATTR_STMT_TYPE,errhp));
	}
	DEBUG(DLEV_DEBUG,"execute(): stmtType=%d\n",stmtType);
	if(stmtType==OCI_STMT_SELECT) {
		describe();
		define();
	}
}

int Cursor::prepare(const string &sql) {
	DEBUG(DLEV_DEBUG,"Preparing statement\n");
	CHECKERR(errhp,OCIStmtPrepare(stmthp, errhp,(text *) sql.c_str() ,(ub4) strlen(sql.c_str()), (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT));	
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &stmtType,(ub4 *) 0,OCI_ATTR_STMT_TYPE,errhp));
	return (int)stmtType;
}

void Cursor::exec() {
	DEBUG(DLEV_DEBUG,"Executing\n");
	if(stmtType==OCI_STMT_SELECT) {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 0,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	} else {
		CHECKERR(errhp,OCIStmtExecute(svchp, stmthp,errhp,(ub4) 1,(ub4) 0, NULL, NULL, OCI_DEFAULT));
	}
}

void Cursor::describe() {
	DEBUG(DLEV_DEBUG,"Describing\n");
	CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,(dvoid *) &nCols,(ub4 *) 0,OCI_ATTR_PARAM_COUNT,errhp));
	row=new (OraType *)[nCols];
	for(int col=0;col<nCols;col++) {
		newCellByType(&(row[col]),stmthp,col);
	}
	if(nCols) haveResultSet=1;
	hashHead();
}
void Cursor::hashHead() {
	DEBUG(DLEV_DEBUG,"Mapping cols\n");
	for(int col=0;col<nCols;col++) {
		string attrName;
		getColName(col,attrName);
		cols_map[attrName]=col;
	}
}

void Cursor::define() {
	DEBUG(DLEV_DEBUG,"Defining output variables\n");
	for(int i=0;i<nCols;i++) {
		row[i]->define(stmthp,i);
	}
	haveResultSet=1;
}	
/* Binding func's */
void Cursor::bind(const string &par,char *buf,int buflen,short *isNull=0) {
	binds[par]=0;
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 (dvoid *)buf,(sb4)buflen,SQLT_STR,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=%s\n",par.c_str(),buf);
}
void Cursor::bind(const string &par,const string &val,short *isNull=0) {
	binds[par]=0;
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 (dvoid *)val.c_str(),(sb4)strlen(val.c_str())+1,SQLT_STR,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=%s\n",par.c_str(),val.c_str());
}

void Cursor::bind(const string &par,int &val,short *isNull=0) {
	binds[par]=0;
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 (dvoid *)&val,(sb4)sizeof(int),SQLT_INT,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=%d\n",par.c_str(),val);
}

void Cursor::bind(const string &par,double &val,short *isNull=0) {	
	binds[par]=0;
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 (dvoid *)&val,(sb4)sizeof(double),SQLT_FLT,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=%f\n",par.c_str(),val);
}

void Cursor::bind(const string &par,RowID &rowid,short *isNull=0) {
	binds[par]=0;
	OraRowID *rid=rowid.getRowID();
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 &(rid->rowid[curRow]),-1,SQLT_RDD,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=ROWID\n",par.c_str());
}

void Cursor::bind(const string &par,Cursor &cur,short *isNull=0) {
	binds[par]=0;
	cur.init(envhp,svchp,errhp,1,REFCURSOR);
	CHECKERR(errhp,OCIBindByName(stmthp,&(binds[par]),errhp,(CONST text *)par.c_str(),(sb4)strlen(par.c_str()),
								 (dvoid *)&(cur.stmthp),(sb4)0,SQLT_RSET,(dvoid *)isNull,(ub2 *)0,
								 (ub2 *)0,(ub4)0,(ub4 *)0,OCI_DEFAULT));
	DEBUG(DLEV_DEBUG,"Bind done.param=%s,val=Cursor\n",par.c_str());
}	
/* --------------------------------------*/

bool Cursor::fetch() {
	unsigned tmp_fetched;
	canFetch=1;
	if(haveResultSet) {
		curRow++;
		if(curRow==fetched && fetched<prefetchRows) {
			canFetch=0;
		} else if(curRow==fetched || !fetched) {
			status=OCIStmtFetch(stmthp,errhp,(ub4) prefetchRows,OCI_FETCH_NEXT,OCI_DEFAULT); 
			tmp_fetched=fetched_all;
			CHECKERR(errhp,OCIAttrGet(stmthp,OCI_HTYPE_STMT,&fetched_all,0,OCI_ATTR_ROW_COUNT,errhp));
			fetched=fetched_all-tmp_fetched;
			DEBUG(DLEV_DEBUG,"fetched=%d\n",fetched);
			if(!fetched) {
				canFetch=0;
				DEBUG(DLEV_DEBUG,"No more rows\n");
			} else {
				canFetch=1;
				curRow=0;
			} 
		} 
	}
	return (canFetch==1)?true:false;
}

/* 	get<Type> functions */
void Cursor::getStr(int col,string &str,bool *isNull) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->isNull(curRow)) {
		str.assign(nulltext);
		if (isNull) *isNull = true;
		return;
	}
	row[col]->getStr(str,curRow);
	if (isNull) *isNull = false;
}

void Cursor::getStr(const string &colName,string &val,bool *isNull) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getStr(iter->second,val,isNull);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getInt(int col,int &val,bool *isNull,const int *null_value) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_NUM) throw OraError("OCICPPLIB: Trying to get Int from non-number column",OCICPPERROR);
	if(row[col]->isNull(curRow)) {
		if (isNull) *isNull = true;
		val = null_value ? *null_value : 0;
    } else {
		if (isNull) *isNull = false;
		((OraNumber *)row[col])->getInt(val,curRow);
	}
}

void Cursor::getInt(const string &colName,int &val,bool *isNull,const int *null_value) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getInt(iter->second,val,isNull,null_value);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getDouble(int col,double &val,bool *isNull,const double *null_value) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_NUM) throw OraError("OCICPPLIB: Trying to get Double from non-number column",OCICPPERROR);
	if(row[col]->isNull(curRow)) {
		if (isNull) *isNull = true;
		val = null_value ? *null_value: 0;
	} else {
		if (isNull) *isNull = false;
			((OraNumber *)row[col])->getDouble(val,curRow);
	}
}

void Cursor::getDouble(const string &colName,double &val,bool *isNull,const double *null_value) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getDouble(iter->second,val,isNull,null_value);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

/* date-related functions ------------------------------------------------------------ */
void Cursor::getStrMon(int col,string &mon) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) mon="";
	else ((OraDate *)row[col])->getStrMon(mon,curRow);
}

void Cursor::getStrMon(const string &colName,string &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getStrMon(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getSec(int col,int &sec) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) sec=0;
	else ((OraDate *)row[col])->getSec(sec,curRow);
}

void Cursor::getSec(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getSec(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getMin(int col,int &min) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) min=0;
	else ((OraDate *)row[col])->getMin(min,curRow);
}
void Cursor::getMin(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getMin(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getHour(int col,int &hour) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) hour=0;
	else ((OraDate *)row[col])->getHour(hour,curRow);
}

void Cursor::getHour(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getHour(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getDay(int col,int &day) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) day=0;
	else ((OraDate *)row[col])->getDay(day,curRow);
}

void Cursor::getDay(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getDay(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getMonth(int col,int &mon) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) mon=0;
	else ((OraDate *)row[col])->getMonth(mon,curRow);
}
void Cursor::getMonth(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getMonth(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}
void Cursor::getYear(int col,int &year) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_DAT) throw OraError("OCICPPLIB: Trying to get Month from non-date column",OCICPPERROR);
	if(row[col]->isNull(curRow)) year=0;
	else ((OraDate *)row[col])->getYear(year,curRow);
}
void Cursor::getYear(const string &colName,int &val) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getYear(iter->second,val);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

/* lob-related functions --------------------------------------------- */
void Cursor::getCLOB(int col,Lob &lob) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_CLOB) throw OraError("OCICPPLIB: Trying to get CLOB from non-CLOB column",OCICPPERROR);
	((OraLob *)row[col])->getCLOB(lob,curRow); /* It's because lob's operation needs Server Context as one of it arguments */ 
}

void Cursor::getCLOB(const string &colName,Lob &lob) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		DEBUG(DLEV_DEBUG,"Fetching CLOB\n");
		getCLOB(iter->second,lob);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getBLOB(const int col,Lob &lob) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_BLOB) throw OraError("OCICPPLIB: Trying to get BLOB from non-BLOB column",OCICPPERROR);
	((OraLob *)row[col])->getBLOB(lob,curRow);
}

void Cursor::getBLOB(const string &colName,Lob &lob) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getBLOB(iter->second,lob);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getFILE(const int col,BFile &bfile) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_FILE) throw OraError("OCICPPLIB: Trying to get BFILE from non-BFILE column",OCICPPERROR);
	((OraBFile *)row[col])->getFILE(bfile,curRow);
}

void Cursor::getFILE(const string &colName,BFile &bfile) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getFILE(iter->second,bfile);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}


/* RowID related functions ----------------------------------------*/
void Cursor::getRowID(int col,RowID &rowid) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_RDD) throw OraError("OCICPPLIB: Trying to get RowID from non-RowID column",OCICPPERROR);
	rowid.init((OraRowID *)row[col]); 
}

void Cursor::getRowID(const string &colName,RowID &rowid) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getRowID(iter->second,rowid);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

/* REFCURSOR related functions -------------------------------------*/
void Cursor::getCursor(int col,Cursor &cur) const {
	if(!canFetch || col<0 || col>=nCols) throw OraError(CELL_NOT_EXISTS);
	if(row[col]->getType()!=SQLT_RSET) throw OraError("OCICPPLIB: Trying to get CURSOR from non-CURSOR column",OCICPPERROR);	
	cur.drop();
	cur.init(envhp,svchp,errhp,1,NTABLE);
	cur.stmthp=((OraRefCur *)row[col])->getStmt(curRow);
}

void Cursor::getCursor(const string &colName,Cursor &cur) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getCursor(iter->second,cur);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

/* ----------------------*/


int Cursor::getNCols() const {
	if(haveResultSet) return (int)nCols;
	throw OraError("OCICPPLIB: Can't return number of columns: No result set for this cursor",OCICPPERROR);
}	

void Cursor::getColName(int col,string &ret_col) const {
	if(!haveResultSet) throw OraError("OCICPPLIB: Cannot get column name: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw OraError("OCICPPLIB: Cannot get column name: column number is out of range",OCICPPERROR);
	row[col]->getAttrName(ret_col);
}

int Cursor::getColSize(int col) const {
	if(!haveResultSet) throw OraError("OCICPPLIB: Cannot get column name: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw OraError("OCICPPLIB: Cannot get column name: column number is out of range",OCICPPERROR);
	return row[col]->getSize();
}

int Cursor::getColSize(const string &colName) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		return getColSize(iter->second);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}	
}
int Cursor::getColType(int col) const {
	if(!haveResultSet) throw OraError("OCICPPLIB: Cannot get column type: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw OraError("OCICPPLIB: Cannot get column type: column number is out of range",OCICPPERROR);	
	return row[col]->getType();
}
int Cursor::getColType(const string &colName) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		return getColType(iter->second);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

void Cursor::getColTypeName(int col,string &typeName) const {
	if(!haveResultSet) throw OraError("OCICPPLIB: Cannot get column type name: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw OraError("OCICPPLIB: Cannot get column type name: column number is out of range",OCICPPERROR);	
	row[col]->getTypeName(typeName);
}

void Cursor::getColTypeName(const string &colName,string &typeName) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		getColTypeName(iter->second,typeName);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

bool Cursor::isNull(int col) const {
	if(!haveResultSet) throw OraError("OCICPPLIB: No result set for this cursor",OCICPPERROR);
	if(col<0 || col>=nCols) throw OraError("OCICPPLIB: Column number is out of range",OCICPPERROR);
	return row[col]->isNull(curRow);	
}

bool Cursor::isNull(const string &colName) const {
	map<string,int>::const_iterator iter=cols_map.find(colName);
	if(iter!=cols_map.end()) {
		return isNull(iter->second);
	} else {
		throw OraError(CELL_NOT_EXISTS);
	}
}

/* Set's text which will be returned on request of null cell default "" (empty text) */	
void Cursor::setNullText(const string &nulltext) {
	this->nulltext.assign(nulltext);
}

void Cursor::newCellByType(OraType **cell,OCIStmt *stmt,int col) {
	/* Get type */
	ub2 colType;
	OCIParam *paramd;
	CHECKERR(errhp,OCIParamGet(stmthp,OCI_HTYPE_STMT,errhp,(dvoid **) &paramd,col+1));
	CHECKERR(errhp,OCIAttrGet((dvoid *) paramd,OCI_DTYPE_PARAM,(dvoid *) &colType,(ub4 *) 0,(ub4) OCI_ATTR_DATA_TYPE,errhp));
	switch(colType) {
		/* String variables */
		case SQLT_CHR:
		case SQLT_LNG:
		case SQLT_AFC:
			*cell=new OraString(envhp,errhp,paramd,colType,prefetchRows);
			break;
		/* Raw & Long Raw */
		case SQLT_BIN:
		case SQLT_LBI:
			*cell=new OraRaw(envhp,errhp,paramd,colType,prefetchRows);
			break;
		/* MLSLABEL */
		case SQLT_LAB:
			*cell=new OraLabel(envhp,errhp,paramd,colType,prefetchRows);
			break;
		/* Numbers */
		case SQLT_NUM:
			*cell=new OraNumber(envhp,errhp,paramd,colType,prefetchRows);
			break;
		/* ROWID */
		case SQLT_RID:
		case SQLT_RDD:
			*cell=new OraRowID(envhp,errhp,paramd,colType,prefetchRows);
			break;
		/* Date */
		case SQLT_DAT:
			*cell=new OraDate(envhp,errhp,paramd,colType,prefetchRows);
			break;
		case SQLT_CLOB:
		case SQLT_BLOB:
			*cell=new OraLob(envhp,errhp,paramd,svchp,colType,prefetchRows);
			break;
		case SQLT_FILE:
			*cell=new OraBFile(envhp,errhp,paramd,svchp,colType,prefetchRows);
			break;
		/* REF CURSORS (nested tables) */
		case SQLT_RSET:
			*cell=new OraRefCur(envhp,errhp,paramd,colType,prefetchRows);
			break;
		case SQLT_NTY:
		case SQLT_REF:
		default:
			*cell=new OraDummy(envhp,errhp,paramd,colType,prefetchRows);
			break;
	}
}
			
